#!/usr/bin/perl
# $Id: hellosign.pl,v 1.25 2016/03/24 13:27:12 rp Exp $
#
# PDFlib PLOP DS:
# generate a document in memory with PDFlib and sign it with PLOP DS

use PDFlib::PLOP 5.1;
use PDFlib::PDFlib 9.0;

my $doc;
my $font;
my $contents;

# Basic signature options
$sign_opts =
    "ltv=try " .         # use ltv=full to ensure LTV status
    # when using PKCS11 use
    # "engine=pkcs#11 digitalid={filename=cryptoki.dll} "
    "engine=builtin digitalid={filename=demo_signer_rsa_2048.p12} password={demo} " ;

# This is where input files live. Adjust as necessary.
my $searchpath = "../data";

my $out_filename = "hellosign.pdf";

# ------------- Step 1: generate the document in memory with PDFlib ------------
my $p = new PDFlib::PDFlib;

eval {
    # create a new PDF document in memory

    $p->begin_document("", "");

    $p->set_info("Creator", "hellosign.pl");
    $p->set_info("Author", "Rainer Plckl");
    $p->set_info("Title", "Hello, world/signed (Perl)!");

    $font = $p->load_font("Helvetica-Bold", "host", "");

    $p->begin_page_ext(595, 842, "");

    $p->setfont($font, 24);
    $p->set_text_pos(50, 700);
    $p->show("Hello, world/signed!");
    $p->continue_text("(says Perl)");
    $p->end_page_ext("");

    $p->end_document("");

    # fetch the buffer containing the PDF document
    $contents = $p->get_buffer();
};

if ($@) {
    die("$0: PDFlib Exception occurred:\n$@");
}

# ------------------ Step 2: sign the document -----------------
    
# create a new plop object
my $plop = new PDFlib::PLOP;

eval {
    $plop->set_option("searchpath={${searchpath}}");

    # Create a virtual PVF file from the document data in memory
    $plop->create_pvf("/pvf/hello.pdf", $contents, "");

    # open unsigned input document from PVF file
    if (($doc = $plop->open_document("/pvf/hello.pdf", "")) == -1)
    {
	die("Error: ", $plop->get_apiname(), ": ", $plop->get_errmsg(), "\n");
    }
    
    if ($plop->prepare_signature($sign_opts) == -1)
    {
        die("Error: ", $plop->get_apiname(), ": ", $plop->get_errmsg(), "\n");
    }

    # sign the output file
    my $optlist = sprintf("input=%d", $doc);
    if ($plop->create_document($out_filename, $optlist) == -1)
    {
	die("Error: ", $plop->get_apiname(), ": ", $plop->get_errmsg(), "\n");
    }

    # close the input document
    $plop->close_document($doc, "");

    # delete PVF file containing input document
    if ($plop->delete_pvf("/pvf/hello.pdf") == -1)
    {
	die("Error: ", $plop->get_apiname(), ": ", $plop->get_errmsg(), "\n");
    }
};

if ($@) {
    die("$0: PLOP Exception occurred:\n$@");
}
