# PLOP sample application for dumping PDF information with PLOP
#
# $Id: dumper.py,v 1.14 2016/04/29 13:00:32 rp Exp $
#

import sys
from sys import *
from traceback import print_tb
from PDFlib.PLOP import *

def yesno(arg):
    if (arg != 0):
        return "yes"
    return "no"


if len(argv) != 2:
    raise Exception("usage: dumper <filename>")

# create a new PLOP object
plop = PLOP()

try:
    docoptlist = "requiredmode=minimum"
    globaloptlist = ""
    searchpath = "../data"

    plop.set_option(globaloptlist)
    plop.set_option("searchpath={" + searchpath + "}")

    doc = plop.open_document(argv[1], docoptlist)

    if doc  == -1:
        raise Exception("ERROR: %s\n" % plop.get_errmsg())

    # --------- general information (always available) */
    pcosmode = int(plop.pcos_get_number(doc, "pcosmode"))

    sys.stdout.write(("       File name: %s\n" % \
    plop.pcos_get_string(doc, "filename")))

    sys.stdout.write(("     PDF version: %s\n" % \
        plop.pcos_get_string(doc, "pdfversionstring")))

    sys.stdout.write(("      Encryption: %s\n" % \
        plop.pcos_get_string(doc, "encrypt/description")))
    recipientcount = \
                int(plop.pcos_get_number( doc, "length:encrypt/recipients"))
    if recipientcount > 0:
        sys.stdout.write("Recipient groups: %d\n" % recipientcount);
        sys.stdout.write("      Digital ID: ");

        if pcosmode == 2:
            sys.stdout.write("supplied digital ID has master permissions\n")
        elif pcosmode == 1:
            sys.stdout.write("supplied digital ID doesn't have master permissions\n")
        else:
            sys.stdout.write("no suitable digital ID supplied\n")

    sys.stdout.write(("       Master pw: %s\n" % \
        yesno(plop.pcos_get_number(doc, "encrypt/master"))))

    sys.stdout.write(("         User pw: %s\n" % \
        yesno(plop.pcos_get_number(doc, "encrypt/user"))))

    sys.stdout.write(("    Text copying: %s\n" % \
        yesno(not plop.pcos_get_number(doc, "encrypt/nocopy"))))

    sys.stdout.write(("      Linearized: %s\n" % \
        yesno(plop.pcos_get_number(doc, "linearized"))))

    if (pcosmode == 0):
        sys.stdout.write("    Minimum mode: no more information available\n")
        plop.close_document(doc, "")
        exit(0)

    # --------- more details (requires at least user password) */

    sys.stdout.write(("    PDF/X status: %s\n" % plop.pcos_get_string(doc, "pdfx")))

    sys.stdout.write(("    PDF/A status: %s\n" % plop.pcos_get_string(doc, "pdfa")))

    sys.stdout.write(("   PDF/UA status: %s\n" % plop.pcos_get_string(doc, "pdfua")))

    sys.stdout.write(("   PDF/VT status: %s\n" % plop.pcos_get_string(doc, "pdfvt")))

    sys.stdout.write("        XFA data: %s\n" % \
        yesno(plop.pcos_get_number(doc, "type:/Root/AcroForm/XFA") != 0))

    sys.stdout.write(("      Tagged PDF: %s\n" % \
        yesno(plop.pcos_get_number(doc, "tagged"))))

    # --------- analyze digital signatures 

    count = int(plop.pcos_get_number(doc, "length:signaturefields"))

    for i in range(count):
        sys.stdout.write("   Sig. field #%d: '%s'," % (i+1, plop.pcos_get_string(doc, "signaturefields[%d]/fullname" % i)))

        if (plop.pcos_get_number(doc, "signaturefields[%d]/visible" % i)):
            sys.stdout.write(" visible")
        else:
            sys.stdout.write(" invisible")

        sigtype = plop.pcos_get_string(doc, "signaturefields[%d]/sigtype" % i)

        if (sigtype == "none"):
            sys.stdout.write(", unsigned")
            continue

        sys.stdout.write(" %s signature" % sigtype)

        if (sigtype == "certification"):
            sys.stdout.write(" (%s allowed)" % plop.pcos_get_string(doc,
                    "signaturefields[%s]/permissions" % i))

        if (plop.pcos_get_number(doc, "signaturefields[%d]/cades" % i)):
            sys.stdout.write(", CAdES")

        sys.stdout.write("\n")

    if (count > 0):
        sys.stdout.write("No. of revisions: %d\n" % plop.pcos_get_number(doc, "revisions"))

    sys.stdout.write("  Reader-enabled: %s\n" % plop.pcos_get_string(doc, "usagerights"))

    sys.stdout.write(("    No. of pages: %d\n" % \
        int(plop.pcos_get_number(doc, "length:pages"))))

    sys.stdout.write(("     Page 1 size: width=%g, height=%g\n" % \
        (plop.pcos_get_number(doc, "pages[%d]/width" % 0),
         plop.pcos_get_number(doc, "pages[%d]/height" % 0))))

    count = int(plop.pcos_get_number(doc, "length:fonts"))
    sys.stdout.write(("    No. of fonts: %d\n" % count))

    for i in range(count):
        type = plop.pcos_get_string(doc, "fonts[%d]/type" % i)
        name = plop.pcos_get_string(doc, "fonts[%d]/name" % i)

        if (plop.pcos_get_number(doc, "fonts[%d]/embedded" % i)):
            sys.stdout.write("embedded %s font %s\n" % (type, name))
        else:
            sys.stdout.write("unembedded %s font %s\n" % (type, name))

    sys.stdout.write("\n")

    plainmetadata = \
            int(plop.pcos_get_number(doc, "encrypt/plainmetadata"))

    if (pcosmode == 1 and not plainmetadata and \
            int(plop.pcos_get_number(doc, "encrypt/nocopy"))):
        sys.stdout.write("Restricted mode: no more information available\n")
        plop.close_document(doc, "")
        exit(0)

    # --------- document info keys and XMP metadata (requires master pw
    # or plaintext metadata)
    #

    count = int(plop.pcos_get_number(doc, "length:/Info"))

    for i in range(0, count):
        objtype = int(plop.pcos_get_number(doc, "type:/Info[%d]" % i))
        key = plop.pcos_get_string(doc, "/Info[%d].key" % i)

        # Info entries can be stored as string or name objects */
        # plop_to_sting == 4; plop_ot_name == 3
        if (objtype == 4 or objtype == 3):
            sys.stdout.write("%16s: '%10s'\n" % \
                (key, plop.pcos_get_string(doc, "/Info[%d]" % i)))
        else:
            sys.stdout.write("%16s: (%s object)\n" % \
               (key, plop.pcos_get_string(doc, "type:/Info[%d]" % i)))

    sys.stdout.write("\n")

    objtype = int(plop.pcos_get_number(doc, "type:/Root/Metadata"))
    # plop_ot_stream == 7
    if (objtype == 7):
        contents = plop.pcos_get_stream(doc, "", "/Root/Metadata")
        ustring = plop.convert_to_unicode("utf8", contents, "outputformat=utf16")
        sys.stdout.write("   XMP meta data: %d bytes (%d Unicode characters)\n" % 
            (len(contents), len(ustring)/2))
    else:
        sys.stdout.write("   XMP meta data: not present\n")

    plop.close_document(doc, "")

except PLOPException:
    sys.stdout.write("PDFlib PLOP exception occurred:\n[%d] %s: %s\n" %
        ((plop.get_errnum()), plop.get_apiname(), plop.get_errmsg()))
    sys.stdout.write(exc_info()[2])

except Exception:
    sys.stdout.write("Exception occurred: %s\n" % (exc_info()[0]))
    print_tb(exc_info()[2])

finally:
    plop.delete()
